@@ -81,15 +81,15 @@ impl Dbc {
 mod tests {
     use super::*;
 
-    use std::collections::{BTreeSet, HashMap};
+    use std::collections::BTreeSet;
     use std::iter::FromIterator;
 
     use quickcheck_macros::quickcheck;
 
     use crate::tests::{NonZeroTinyInt, TinyInt};
     use crate::{
-        Amount, DbcBuilder, DbcHelper, Hash, KeyManager, Mint, ReissueRequest, SimpleKeyManager,
-        SimpleSigner, SimpleSpendBook,
+        Amount, DbcBuilder, DbcHelper, Hash, KeyManager, MintNode, ReissueRequest,
+        ReissueRequestBuilder, SimpleKeyManager, SimpleSigner, SpentProof, SpentProofShare,
     };
 
     fn divide(amount: Amount, n_ways: u8) -> impl Iterator<Item = Amount> {
@@ -106,8 +106,9 @@ mod tests {
         dbc_owner: &bls_dkg::outcome::Outcome,
         dbc: Dbc,
         n_ways: u8,
-        output_owner: &blsttc::PublicKeySet,
-    ) -> Result<ReissueRequest, Error> {
+        output_owner: PublicKey,
+        spentbook_node: &MintNode<SimpleKeyManager>,
+    ) -> Result<ReissueRequest> {
         let amount_secrets = DbcHelper::decrypt_amount_secrets(dbc_owner, &dbc.content)?;
 
         let reissue_tx = crate::TransactionBuilder::default()
@@ -115,27 +116,37 @@ mod tests {
             .add_outputs(
                 divide(amount_secrets.amount, n_ways).map(|amount| crate::Output {
                     amount,
-                    owner: output_owner.public_key(),
+                    owner: output_owner,
                 }),
             )
             .build()?;
 
-        let sig_share = dbc_owner
-            .secret_key_share
-            .derive_child(&dbc.spend_key_index())
-            .sign(&reissue_tx.blinded().hash());
+        let tx_hash = reissue_tx.blinded().hash();
 
-        let sig = dbc_owner
-            .public_key_set
-            .combine_signatures(vec![(dbc_owner.index, &sig_share)])
-            .unwrap();
+        let spent_sig = dbc_owner.public_key_set.combine_signatures([(
+            dbc_owner.index,
+            dbc_owner
+                .secret_key_share
+                .derive_child(&dbc.spend_key_index())
+                .sign(tx_hash),
+        )])?;
+        let spentbook_pks = spentbook_node.key_manager.public_key_set()?;
+        let spentbook_sig_share = spentbook_node
+            .key_manager
+            .sign(&SpentProof::proof_msg(&tx_hash, &spent_sig))?;
 
-        let request = ReissueRequest {
-            transaction: reissue_tx,
-            input_ownership_proofs: HashMap::from_iter([(dbc.spend_key(), sig)]),
-        };
+        let rr = ReissueRequestBuilder::new(reissue_tx)
+            .add_spent_proof_share(
+                dbc.spend_key(),
+                SpentProofShare {
+                    spent_sig,
+                    spentbook_pks,
+                    spentbook_sig_share,
+                },
+            )
+            .build()?;
 
-        Ok(request)
+        Ok(rr)
     }
 
     #[test]
@@ -192,18 +203,18 @@ mod tests {
             SimpleSigner::from(genesis_owner.clone()),
             genesis_owner.public_key_set.public_key(),
         );
-        let mut genesis_node = Mint::new(key_manager, SimpleSpendBook::new());
+        let mut genesis_node = MintNode::new(key_manager);
 
-        let (gen_dbc_content, gen_dbc_trans, (gen_key_set, gen_node_sig)) =
-            genesis_node.issue_genesis_dbc(amount).unwrap();
+        let genesis = genesis_node.issue_genesis_dbc(amount).unwrap();
 
-        let genesis_sig = gen_key_set
-            .combine_signatures(vec![gen_node_sig.threshold_crypto()])
+        let genesis_sig = genesis
+            .public_key_set
+            .combine_signatures(vec![genesis.transaction_sig.threshold_crypto()])
             .unwrap();
 
         let genesis_dbc = Dbc {
-            content: gen_dbc_content,
-            transaction: gen_dbc_trans,
+            content: genesis.dbc_content,
+            transaction: genesis.transaction,
             transaction_sigs: BTreeMap::from_iter([(
                 crate::genesis_dbc_input(),
                 (genesis_key, genesis_sig),
@@ -215,15 +226,11 @@ mod tests {
             &genesis_owner,
             genesis_dbc,
             n_inputs.coerce(),
-            &input_owner.public_key_set,
+            input_owner.public_key_set.public_key(),
+            &genesis_node,
         )?;
 
-        let split_reissue_share = genesis_node
-            .reissue(
-                reissue_request.clone(),
-                reissue_request.transaction.blinded().inputs,
-            )
-            .unwrap();
+        let split_reissue_share = genesis_node.reissue(reissue_request.clone()).unwrap();
 
         let mut dbc_builder = DbcBuilder::new(reissue_request.transaction);
         dbc_builder = dbc_builder.add_reissue_share(split_reissue_share);
@@ -244,35 +251,35 @@ mod tests {
             })
             .build()?;
 
-        let input_ownership_proofs = HashMap::from_iter(reissue_tx.inputs.iter().map(|input| {
-            let sig_share = input_owner
-                .secret_key_share
-                .derive_child(&input.spend_key_index())
-                .sign(&reissue_tx.blinded().hash());
+        let tx_blinded = reissue_tx.blinded();
 
-            let sig = input_owner
-                .public_key_set
-                .combine_signatures(vec![(input_owner.index, &sig_share)])
-                .unwrap();
+        let mut rr_builder = ReissueRequestBuilder::new(reissue_tx.clone());
+        for input in reissue_tx.inputs.iter() {
+            let spent_sig = input_owner.public_key_set.combine_signatures([(
+                input_owner.index,
+                input_owner
+                    .secret_key_share
+                    .derive_child(&input.spend_key_index())
+                    .sign(tx_blinded.hash()),
+            )])?;
+            let spentbook_pks = genesis_node.key_manager.public_key_set()?;
+            let spentbook_sig_share = genesis_node
+                .key_manager
+                .sign(&SpentProof::proof_msg(&tx_blinded.hash(), &spent_sig))?;
 
-            (input.spend_key(), sig)
-        }));
+            rr_builder = rr_builder.add_spent_proof_share(
+                input.spend_key(),
+                SpentProofShare {
+                    spent_sig,
+                    spentbook_pks,
+                    spentbook_sig_share,
+                },
+            );
+        }
 
-        let reissue_request = ReissueRequest {
-            transaction: reissue_tx,
-            input_ownership_proofs,
-        };
-
-        let reissue_share = genesis_node
-            .reissue(
-                reissue_request.clone(),
-                reissue_request.transaction.blinded().inputs,
-            )
-            .unwrap();
-        assert_eq!(
-            reissue_request.transaction.blinded(),
-            reissue_share.dbc_transaction
-        );
+        let rr = rr_builder.build()?;
+        let reissue_share = genesis_node.reissue(rr).unwrap();
+        assert_eq!(tx_blinded, reissue_share.dbc_transaction);
 
         let (mint_key_set, mint_sig_share) =
             reissue_share.mint_node_signatures.values().next().unwrap();
@@ -313,8 +320,7 @@ mod tests {
                 .take(n_valid_sigs.coerce())
                 .map(|in_owner| (*in_owner, (genesis_key, mint_sig.clone()))),
         );
-        let mut repeating_inputs = reissue_request
-            .transaction
+        let mut repeating_inputs = reissue_tx
             .inputs
             .iter()
             .cycle()
